package haoqipei.fl_update_apk_pl

import android.content.Context
import android.widget.Toast
import androidx.annotation.NonNull
import com.allenliu.versionchecklib.callback.OnCancelListener
import com.allenliu.versionchecklib.v2.AllenVersionChecker
import com.allenliu.versionchecklib.v2.builder.DownloadBuilder
import com.allenliu.versionchecklib.v2.builder.NotificationBuilder
import com.allenliu.versionchecklib.v2.builder.UIData
import com.allenliu.versionchecklib.v2.callback.ForceUpdateListener
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result

/** FlUpdateApkPlPlugin */
public class FlUpdateApkPlPlugin : FlutterPlugin, MethodCallHandler {
    private var methodChannel: MethodChannel? = null
    private lateinit var builder: DownloadBuilder
    private var context: Context? = null
    private lateinit var downloadDataEntity: DownloadDataEntity
    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        context = flutterPluginBinding.applicationContext
        //通信方法注册
        methodChannel = MethodChannel(flutterPluginBinding.binaryMessenger, "fl_update_apk_pl").apply {
            setMethodCallHandler(this@FlUpdateApkPlPlugin)
        }
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        if (call.method == "sendDownloadData") {
            val params = call.arguments<Map<String, String>>()
            downloadDataEntity = DownloadDataEntity(params["requestUrl"], params["title"], params["content"], params["forcesUpdate"])
            if (params["requestUrl"].isNullOrEmpty()) {
                return Toast.makeText(context, "下载地址为空", Toast.LENGTH_SHORT).show()
            }
            sendRequest(downloadDataEntity)
        } else {
            result.notImplemented()
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        methodChannel?.setMethodCallHandler(null)
        methodChannel = null
        //合适的地方关闭
        AllenVersionChecker.getInstance().cancelAllMission()
        builder?.destory()
    }

    //开始下载
    private fun sendRequest(downloadDataEntity: DownloadDataEntity) {
        builder = AllenVersionChecker.getInstance().downloadOnly(crateUIData())
        builder.isShowNotification = true
        builder.isForceRedownload = true
        if (downloadDataEntity.forcesUpdate.equals("1")) {
            builder.forceUpdateListener = ForceUpdateListener {
                Toast.makeText(context, "下载完成", Toast.LENGTH_SHORT).show()
            }
        }
        builder.notificationBuilder = NotificationBuilder.create()
                .setRingtone(true)
                .setContentTitle(downloadDataEntity.title)
                .setContentText(downloadDataEntity.content)
        builder.executeMission(context)
    }

    private fun crateUIData(): UIData? {
        val uiData = UIData.create()
        uiData.title = downloadDataEntity.title
        uiData.downloadUrl = downloadDataEntity.requestUrl
        uiData.content = downloadDataEntity.content
        return uiData
    }
}
